package meerkat.bulletinboard;

import meerkat.comm.*;
import static meerkat.protobuf.BulletinBoardAPI.*;

import java.util.List;

/**
 * Created by talm on 24/10/15.
 */
public interface BulletinBoardClient {

    /**
     * Initialize the client to use some specified servers
     * @param meerkatDBs is the list of database locations
     */
    void init(List<String> meerkatDBs);

    /**
     * Post a message to the bulletin board
     * @param msg
     */
    MessageID postMessage(BulletinBoardMessage msg) throws CommunicationException;

    /**
     * Check how "safe" a given message is
     * @param id
     * @return a normalized "redundancy score" from 0 (local only) to 1 (fully published)
     */
    float getRedundancy(MessageID id);

    /**
     * Read all messages posted matching the given filter
     * Note that if messages haven't been "fully posted", this might return a different
     * set of messages in different calls. However, messages that are fully posted
     * are guaranteed to be included.
     * @param filterList return only messages that match the filters (null means no filtering).
     * @return
     */
    List<BulletinBoardMessage> readMessages(MessageFilterList filterList);

    interface MessageCallback {
        void handleNewMessage(List<BulletinBoardMessage> msg);
    }

    /**
     * Register a callback that will be called with each new message that is posted.
     * The callback will be called only once for each message.
     * @param callback
     * @param filterList only call back for messages that match the filter.
     */
    void registerNewMessageCallback(MessageCallback callback, MessageFilterList filterList);

}
